from abc import abstractmethod


class SentenceSplitter:

    SEPARATORS = "()[]{}\"'\u05F4\uFF02\u055B"
    SENTENCE_ENDERS = ".?!…"
    PUNCTUATION_CHARACTERS = ",:;"

    @abstractmethod
    def split(self, line: str) -> list:
        pass



from enum import Enum, auto


class WordFormat(Enum):

    """
    Surface/Original form
    """
    SURFACE = auto()
    """
    Create 2-Gram words as output.
    """
    LETTER_2 = auto()
    """
    Create 3-Gram words as output.
    """
    LETTER_3 = auto()
    """
    Create 4-Gram words as output.
    """
    LETTER_ = auto()



from os.path import isfile


class FileDescription:

    __path: str
    __extension: str
    __index: int

    def __init__(self, path: str, extension: str, index: int):
        self.__path = path
        self.__extension = extension
        self.__index = index

    def getPath(self) -> str:
        return self.__path

    def getIndex(self) -> int:
        return self.__index

    def getExtension(self) -> str:
        return self.__extension

    def getFileName(self, thisPath = None, extension = None) -> str:
        if thisPath is None:
            thisPath = self.__path
        return self.getFileNameWithIndex(thisPath, self.__index, extension)

    def getFileNameWithExtension(self, extension: str) -> str:
        return self.getFileName(self.__path, extension)

    def getFileNameWithIndex(self, thisPath: str, index: int, extension = None) -> str:
        if extension is None:
            extension = self.__extension
        return "%s/%04d.%s" % (thisPath, index, extension)

    def getRawFileName(self) -> str:
        return "%04d.%s" % (self.__index, self.__extension)

    def addToIndex(self, count: int):
        self.__index += count

    def nextFileExists(self, count: int, thisPath = None):
        if thisPath is None:
            thisPath = self.__path
        return isfile(self.getFileNameWithIndex(thisPath, self.__index + count))

    def previousFileExists(self, count: int, thisPath = None):
        if thisPath is None:
            thisPath = self.__path
        return isfile(self.getFileNameWithIndex(thisPath, self.__index - count))



from Corpus.Sentence import Sentence


class Paragraph:

    __sentences: list

    """
    A constructor of Paragraph class which creates a list sentences.
    """
    def __init__(self):
        self.__sentences = []

    """
    The addSentence method adds given sentence to sentences list.

    PARAMETERS
    ----------
    s : Sentence
        Sentence type input to add sentences.
    """
    def addSentence(self, s: Sentence):
        self.__sentences.append(s)

    """
    The sentenceCount method finds the size of the list sentences.

    RETURNS
    -------
    int
        The size of the list sentences.
    """
    def sentenceCount(self) -> int:
        return len(self.__sentences)

    """
    The getSentence method finds the sentence from sentences list at given index.

    PARAMETERS
    ----------
    index : int
        used to get a sentence.
        
    RETURNS
    -------
    Sentence
        sentence at given index.
    """
    def getSentence(self, index: int) -> Sentence:
        return self.__sentences[index]
